/// @file        HwAdapt.c
/// @copyright     Copyright (c) 2012 Toradex AG \n
///             All information contained herein is, and remains the property of Toradex Ag and its suppliers, if any. The intellectual\n
///             and technical concepts contained herein are proprietary to Toradex AG and its suppliers and may be covered by Swiss and\n
///             foreign patents, patents in process, and are protected by trade secret or copyright law. Dissemination of this information\n
///             or reproduction of this material is strictly forbidden unless prior written permission is obtained from Toradex AG.\n
/// @author      $Author$
/// @version     $Rev$
/// @date        $Date$
/// @brief       The hardware adaption of the multitouch driver solution is made to run as an application.\n
///             The advantage is that a customer can modify, run and test the "driver" like a normal application. \n
///             The "hardware adaption" handles the whole interface to the touch controller (interrupt, I2C communication etc.)
///             using the standard Toradex library which are free available. Therefore it is much easier for a customer to adapt this
///             "hardware adaption" to other touch controller. \n
///                The counterpart to this "hardware adaption" is the "unified driver" which must be installed like a normal driver. \n
///                The "unified driver" has no connection to the hardware and run on all colibri modules and WinCE versions.
/// @target        Colibri T20, T30, WinEC7
/// @caveats    No screen output. All messages are send to the debug port.\n
///                This file contains the main loop.\n
///
#include <windows.h>
#include <Winbase.h>
#include <Winuser.h>
#include <winioctl.h>
#include <stdio.h>
#include "StdAfx.h"
#include "TchContr_Fusion.h"


//==== Debug Zones ================================================================================
// simple handling: debug zones are forced with TREU or FALSE and need a recompile to be activated
// for sophisticated handling: use macro DEBUGZONE(), DBGPARAM dpCurSettings etc.
// (see MSDN e.g. Win CE 5.0, Debug Zones)
#undef ZONE_ERROR
#undef ZONE_WARN
#undef ZONE_FUNCTION
#undef ZONE_INFO
#undef ZONE_TIPSTATE

#define ZONE_ERROR      TRUE
#define ZONE_WARN       TRUE
#define ZONE_FUNCTION   FALSE
#define ZONE_INFO       FALSE
#define ZONE_TIPSTATE   FALSE

//==== privat preprocessor definitions ============================================================

//==== public extern methods =====================================================================
void vI2C_ReadConfiguration(void);
BOOL bI2C_Init(void);
void vI2C_DeInit(void);
BOOL bInitFusion(void);
BOOL bWaitTouchDataINT(void);
BOOL bCommunicationTestFusion(void);
void vTouchPanelGetPoint(PTCHINPUT pInput,INT *pCnt);
void vDeInitFusion(void);

//=================================================================================================
/// _tmain
int WINAPI WinMain( HINSTANCE hInstance,
                    HINSTANCE hPrevInstance,
                    LPTSTR    lpCmdLine,
                    int       nCmdShow)
{
    HANDLE            hDS;                                                // handle for "unified driver"
    CETCHINP_IOCTL    CeTchInp;                                            // touch events packed for IOCTL
    DWORD            BytesHandled;                                        // dummy for DeviceIoControl()
    BytesHandled    = 0;                                                // to avoid warning

    RETAILMSG(1,(TEXT("MultiTchHwAdapt loading...")));

    // Read the and setup the configuration
    vI2C_ReadConfiguration();                                           // read staic global configuration paramters (default or set in the regitry)

    // Init the IC interface on the Colibri (makes no sense to retry)
    if (FALSE==bI2C_Init())                                             // init I2C communication and setup interrupt handling
    {
        RETAILMSG(1,(TEXT("aborted\r\n")));                                // bI2C_Init() printed the detail of the failure alreday
        vI2C_DeInit();                                                    // release anythings activ allready
        return 0;
    }

    // Get a handle to the driver. Which causes the driver's XXX_Open function to be called
    hDS = CreateFile( TEXT("TCH1:"), GENERIC_READ|GENERIC_WRITE, FILE_SHARE_READ|FILE_SHARE_WRITE, NULL, OPEN_EXISTING, 0, 0);
    if (INVALID_HANDLE_VALUE==hDS)
    {
        RETAILMSG(1,(TEXT("no Unified Touch Driver...")));
        RETAILMSG(1,(TEXT("aborted\r\n")));                                // bI2C_Init() printed the detail of the failure alreday
        vI2C_DeInit();                                                    // release anythings activ allready
        return 0;
    }


    // main loop
    // The used touch controller is setup to set the interrupt line activ if one or more fingers touch the screen.\n
    // If the touch aera is untouched then the interrupt is set to passiv. \n
    // This thread mainly sleeps until a timeout occurs. If this happen the interrupt signal is tested. \n
    // If the interrupt line is activ (touches) then the status, current position etc. are read out\n
    // of the touch controller and are processed. Afterwards the next timeout is set to the sample rate\n
    // and the thread is going to sleep again. \n
    // If the interrupt line is passiv (no touches) then the communcation to the touch controller is checked.\n
    // Afterwards the next timeout is set to the communcation test rate. \n
    // Beside that, a transition of the interrupt line generates a interrupt event which wakup the thread immideatly.\n
    // This shorten up the reaction time for the first touch.

#ifdef TRACE_ON                                                            // Trace on
    TraceTchClear();
#endif


    // main loop
    while (1==1)
    {
        //------ Startup Procedure --------------------------------------------------------------------

        DEBUGMSG(ZONE_INFO,(TEXT("MultiTchHwAdapt: Start communcation with the touch controller Fusion\r\n")));

        if (FALSE==bInitFusion())                                        // reset and start touch controller
        {
            goto restart;
        }

        RETAILMSG(TRUE,(TEXT("done\r\n")));                                // app driver runs

        // wait for messages
        while(1==1)
        {
            if (FALSE==bWaitTouchDataINT())
            {
                #ifdef TRIGGER_ON
                    vSetTriggerHigh();
                #endif

                if (FALSE==bCommunicationTestFusion())                    // timeout, test the communication to teh touch controller
                {
                    goto restart;
                }

#ifdef TRACE_ON                                                            // Trace on
                    TraceTchPrint();

                    Sleep(3000);

                    TraceTchClear();
#endif


            }
            else                                                        // real interrupt
            {
                #ifdef TRIGGER_ON
                    vSetTriggerHigh();
                #endif
                // get the touch events
                vTouchPanelGetPoint(&CeTchInp.TchInp[0],&CeTchInp.iCnt); // get the touch events from the controller

                if ((1==CeTchInp.iCnt)||(2==CeTchInp.iCnt))
                {
					// send the touch events to the unified touch driver
                    DeviceIoControl( hDS, IOCTL_SET_TOUCH_EVENT, &CeTchInp, sizeof(CeTchInp), NULL, 0, &BytesHandled, NULL );
#ifdef TRACE_ON                                                            // Trace on
                    TraceTchPut(&CeTchInp);
#endif
                }
            }
            #ifdef TRIGGER_ON
                vSetTriggerLow();
            #endif
        }

        // restart sequence
restart:
        RETAILMSG(1,(TEXT("MultiTchHwAdapt: restarts communication with touch controller...\r\n")));

        vDeInitFusion();                                                // stop (reset/powerdown) the Touch Controller

        Sleep(400);                                                     // wait before an new try starts
    }
    return 0;
}
